/**************************************************************************
***
***    Copyright (c) 1995-1997 by ABKGroup, UCLA VLSI CAD Laboratory,
***    UCLA Computer Science Department, Los Angeles, CA 90095-1596 USA,
***    and by the Regents of the University of California.
***    All rights reserved.
***
***    No portion of this file may be used, copied, or transmitted for
***    any purpose, including any research or commercial purpose,
***    without the express written consent of Professor Andrew B. Kahng,
***    UCLA Computer Science Department, 3713 Boelter Hall, Los Angeles,
***    CA 90095-1596 USA.   Any copy made of this file must include this
***    notice.    ABKGroup and the Regents of the University of California
***    retain title to all copies of this file and/or its compiled
***    derivatives.
***
***************************************************************************/

// CHANGES

#include "abkcommon.h"
#include "abkversion.h"
#include "DB.h"
#include "capoPlacer.h"
#include "RBPlaceFromDB.h"
#include "hgWDimsFromDB.h"
//#include "greedyOrientOpt.h"
//#include "rowdp.h"
//#include "rownet.h"
//#include "netopt.h"
#include "edgePlEval.h"

int main(int argc, const char *argv[])
{
  NoParams         noParams       (argc,argv);
  BoolParam        helpRequest    ("help",argc,argv);
  BoolParam        helpRequest1   ("h",   argc,argv);
  BoolParam        saveDef        ("save",argc,argv);
  BoolParam        saveOrigDef    ("saveOrig",argc,argv);
  BoolParam        doCellFlipping ("flip",argc,argv);
  BoolParam        doCellSpacing  ("cellSpace",argc,argv);

  Verbosity	   verb(argc, argv);

  PRINT_VERSION_INFO
  cout << CmdLine(argc,argv);

// ------------- Process parameters before getting bogged down in computations

  CapoParameters capoParams(argc,argv);

  if (noParams.found() || helpRequest.found() || helpRequest1.found())
  {
     cerr << "  use '-help' or '-f filename.aux' " << endl;
     return 0;
  }

  capoParams.replaceSmallBlocks=Never;
  cout << capoParams << endl;

  StringParam 	 auxFileName 	("f", 	argc,argv);
  abkfatal(auxFileName.found(),"Usage: prog -f filename.aux <more params>");

  DB db(auxFileName);

  if(saveOrigDef.found())
	db.saveDEF("origDef");

  cerr<<"Design Name: "<<db.getDesignName()<<endl;


  RBPlaceParams rbParam(argc,argv);
  RBPlaceFromDB rbplace(db, rbParam);   

  vector<const char*> hierCellNames;

 {
  Timer capoTimer;
  CapoPlacer capo(rbplace, capoParams);
  capoTimer.stop();

  db.setPlaceAndOrient(rbplace);
       
  cout << "After Capo  "<<endl;
  cout << "  Capo Runtime:             "<<capoTimer.getUserTime()<<endl;

  cout << "  DB   Est Pin-to-Pin HPWL: "<<
	db.getNetlist().evalHalfPerimCost(db.spatial.locations,
				  db.spatial.orientations)<<endl<<endl;

  cout << "  RBPl Est            HPWL: "<<rbplace.evalHPWL(false)<<endl;
  cout << "  RBPl Est Pin-to-Pin HPWL: "<<rbplace.evalHPWL(true)<<endl;;
  cout << "  RBPl Est             WWL: "<<rbplace.evalWeightedWL(false)<<endl;;
  cout << "  RBPl Est Pin-to-Pin  WWL: "<<rbplace.evalWeightedWL(true)<<endl;;

  capo.getHierCellNames(hierCellNames);
 }

//for(unsigned k=0; k!=hierCellNames.size(); k++)
//   cout << setw(6)<< k << ". " << hierCellNames[k]  << endl;
  HGNodeNamesMap      hierNamesMap, tmpNamesMap;
  for(unsigned k=0; k!=hierCellNames.size(); k++)
     hierNamesMap[hierCellNames[k]]=k;

 cout << " ======== Launching another Capo run  --- with hierarchy ========= "
      << endl;

 capoParams.splitterParams.useNameHierarchy=true;
 capoParams.splitterParams.delimiters="_";
 capoParams.replaceSmallBlocks=AtTheEnd;

 
  // now go to RBPlace and swap the old names for the new ones

  HGraphWDimensions&  hg=const_cast<HGraphWDimensions&>(rbplace.getHGraph());
  abkfatal(hg.getNumNodes()==hierCellNames.size(),
            " Numbers of cell names do not match");
  vector<const char*> tmpNames=hg._nodeNames;
  hg._nodeNames=hierCellNames;
  tmpNamesMap=hg._namesMap;
  hg._namesMap =hierNamesMap;

 {
  cout<<capoParams<<endl;

  Timer capoTimer;
  CapoPlacer capo(rbplace, capoParams);
  capoTimer.stop();

  db.setPlaceAndOrient(rbplace);
       
  cout << "After Capo  "<<endl;
  cout << "  Capo Runtime:             "<<capoTimer.getUserTime()<<endl;

  cout << "  DB   Est Pin-to-Pin HPWL: "<<
	db.getNetlist().evalHalfPerimCost(db.spatial.locations,
				  db.spatial.orientations)<<endl<<endl;

  cout << "  RBPl Est            HPWL: "<<rbplace.evalHPWL(false)<<endl;
  cout << "  RBPl Est Pin-to-Pin HPWL: "<<rbplace.evalHPWL(true)<<endl;;
  cout << "  RBPl Est             WWL: "<<rbplace.evalWeightedWL(false)<<endl;;
  cout << "  RBPl Est Pin-to-Pin  WWL: "<<rbplace.evalWeightedWL(true)<<endl;;

 }

  if (saveDef.found()) db.saveDEF("out.def");

/*
  hg._nodeNames=tmpNames;
  hg._namesMap=tmpNamesMap;
*/

  return 0;
}
